# -*- coding: utf-8 -*-
"""
admin for acomea.app.switch_programmato app
"""

from django.utils.translation import ugettext as _
from django.contrib import admin
from django import forms

from jmb.core.admin.options import ExtendibleModelAdmin

from acomea.app.models import Servizio, Applicazione, PosizioneFrom, PosizioneTo


class ServizioAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_edit_icon', 'get_delete_icon',
        'id', 'numero_operazione', 'date_create', 'data_operazione', 'get_status_display', 'profilo', 'codice_cliente',
        'codice_fiscale', 'tipo_operazione',
        'codice_rapporto_from', 'codice_deposito_from', 'codice_fondo_from', 'codice_tipocontratto_from',
        'codice_rapporto_to', 'codice_deposito_to', 'codice_fondo_to', 'codice_tipocontratto_to', 'tipo_piano_to',
        'tipo_qta_movimento', 'mifid',
        'importo', 'numero_rate_piano', 'importo_rata', 'numero_rate_versate', 'quote',
        'iban', 'simulazione',
        'frequenza', 'data_decorrenza', 'fine_servizio', 'fine_servizio_anno', 'fine_servizio_mese',
        'rei_pro_perc_plus',
        'car_die_gruppo_a', 'car_die_gruppo_b', 'car_die_gruppo_a_desc', 'car_die_gruppo_b_desc',
        'car_die_allocazione_a', 'car_die_allocazione_b', 'car_die_perc_var_a', 'car_die_perc_riall_ab',
        'rib_inv_perc_scost', 'dettaglio_from', 'dettaglio_to', 'rad_rib_flag', 'rad_rib_riduz_quota_a',
        'rad_rib_riduz_quota_b', 'rad_rib_nop_pianis', 'messaggio_costo_applicazione', 'costo_applicazione',
    )


class PosizioneFromAdmin(ExtendibleModelAdmin):
    list_display = (
        'servizio', 'codice_rapporto_from', 'codice_deposito_from', 'codice_fondo_from',
        'descrizione_fondo_from', 'codice_tipocontratto_from', 'tipo_piano_from', 'patrimonio_from', 'quote_from',
        'get_edit_icon', 'get_delete_icon',
    )


class PosizioneToAdmin(ExtendibleModelAdmin):
    list_display = (
        'servizio', 'codice_rapporto_to', 'codice_deposito_to', 'codice_fondo_to', 'descrizione_fondo_to',
        'codice_tipocontratto_to', 'tipo_piano_to', 'peso_to'
    )


class ApplicazioneAdmin(ExtendibleModelAdmin):
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (_('general'), {'fields': (
            'status', 'codice', 'description', 'modal_page_slug',
            'from_form', 'sottotitolo_from',
            ('to_form', 'to_form_template_name',),
            ('riepilogo_form', 'riepilogo_form_template_name',),
        )}),
        (_('payment'), {'fields': (
            'data_inizio_pagamento',
            'costo_applicazione',
            'messaggio_gratuito',
            'messaggio_pagamento',
        )}),
    )

    def get_form(self, request, obj=None, **kwargs):
        form = super(ApplicazioneAdmin, self).get_form(request, obj, **kwargs)
        form.base_fields['sottotitolo_from'].widget = forms.Textarea(
            attrs={
                'rows': '5',
            }
        )
        return form

    list_display = (
        'id', 'status', 'codice', 'description',
        'data_inizio_pagamento', 'costo_applicazione',
        'get_codice_display', 'from_form',
        'to_form', 'to_form_template_name',
        'riepilogo_form', 'riepilogo_form_template_name',
        'get_edit_icon', 'get_delete_icon',
    )


admin.site.register(Servizio, ServizioAdmin)
admin.site.register(PosizioneFrom, PosizioneFromAdmin)
admin.site.register(PosizioneTo, PosizioneToAdmin)
admin.site.register(Applicazione, ApplicazioneAdmin)
