# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext as _
from django.conf import settings

from cms.models.pluginmodel import CMSPlugin

from jmb.core.db.cmsmodels import StatusModel


class ServizioFromSettings(CMSPlugin, StatusModel):
    class Meta:
        app_label = "app"

    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.APP_SERVIZIO_CMSPLUGIN_POSIZIONEFROM_FORM_CLASSES,
        default="acomea.app.cms.forms.ServizioFromForm",
    )
    form_template_name = models.CharField(
        _("Form template"),
        max_length=100,
        choices=settings.APP_SERVIZIO_CMSPLUGIN_POSIZIONEFROM_FORM_TEMPLATES,
        default="cms_plugins/posizione_from_form.html"
    )
    unsuccess_slug = models.SlugField(
        _("Unsuccess page"),
        help_text=_("slug pagina insuccesso Posizione From")
    )
    fallback_slug = models.SlugField(
        _("Fallback page"),
        blank=True,
        help_text=_("(Opzionale) slug pagina fallback (in caso di anomalie)")
    )

    def __unicode__(self):
        return "%s %s - Posizione From" % (('On - ' if self.status else 'Off - '), self._meta.app_label)


class ServizioToSettings(CMSPlugin, StatusModel):
    class Meta:
        app_label = "app"

    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.APP_SERVIZIO_CMSPLUGIN_POSIZIONETO_FORM_CLASSES,
        default="acomea.app.cms.forms.ServizioToForm",
    )
    form_template_name = models.CharField(
        _("Form template"),
        max_length=100,
        choices=settings.APP_SERVIZIO_CMSPLUGIN_POSIZIONETO_FORM_TEMPLATES,
        default="cms_plugins/posizione_to_form.html"
    )
    unsuccess_slug = models.SlugField(
        _("Unsuccess page"),
        help_text=_("slug pagina insuccesso Posizione From")
    )
    fallback_slug = models.SlugField(
        _("Fallback page"),
        blank=True,
        help_text=_("(Opzionale) slug pagina fallback (in caso di anomalie)")
    )

    def __unicode__(self):
        return "%s %s - Posizione To" % (('On - ' if self.status else 'Off - '), self._meta.app_label)


class ServizioRiepilogoSettings(CMSPlugin, StatusModel):
    class Meta:
        app_label = "app"

    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.APP_SERVIZIO_CMSPLUGIN_RIEPILOGO_FORM_CLASSES,
        default="acomea.app.cms.forms.ServizioRiepilogoForm",
    )
    form_template_name = models.CharField(
        _("Form template"),
        max_length=100,
        choices=settings.APP_SERVIZIO_CMSPLUGIN_RIEPILOGO_FORM_TEMPLATES,
        default="cms_plugins/riepilogo_form.html"
    )
    unsuccess_slug = models.SlugField(
        _("Unsuccess page"),
        help_text=_("slug pagina insuccesso Riepilogo")
    )
    fallback_slug = models.SlugField(
        _("Fallback page"),
        blank=True,
        help_text=_("(Opzionale) slug pagina fallback (in caso di anomalie)")
    )

    def __unicode__(self):
        return "%s %s - Riepilogo" % (('On - ' if self.status else 'Off - '), self._meta.app_label)


class ServiziInseritiSettings(CMSPlugin, StatusModel):
    class Meta:
        app_label = "app"

    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.APP_SERVIZIO_CMSPLUGIN_SERVIZI_INSERITI_TEMPLATES,
        default="cms_plugins/servizi_inseriti.html"
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.template_name)
