# -*- coding: utf-8 -*-
import datetime

from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from jmb.core.middleware.force_response import ForceResponse
from jmb.core.utils.functions import import_class

from acomea.app.models import Servizio, Applicazione, PosizioneFrom
from acomea.app.cms.models import (
    ServizioFromSettings, ServizioToSettings, ServizioRiepilogoSettings,
    ServiziInseritiSettings
)
from acomea.app.soap import OpServizi


def _reset_position_from_dict(request):
    # questo metodo cancella i dati inseriti in sessione
    # quando si torna indietro
    if 'app_posizioni_intestatario' in request.session:
        del request.session['app_posizioni_intestatario']
    if 'app_posizioni_cointestatario' in request.session:
        del request.session['app_posizioni_cointestatario']
    if 'app_posizioni_to' in request.session:
        del request.session['app_posizioni_to']
    if 'app_dati_servizio' in request.session:
        del request.session['app_dati_servizio']
    if 'app_fondi_to_totali' in request.session:
        del request.session['app_fondi_to_totali']
    if 'app_result' in request.session:
        del request.session['app_result']


def _reset_position_to_dict(request):
    if 'app_posizioni_to' in request.session:
        del request.session['app_posizioni_to']
    if 'app_result' in request.session:
        del request.session['app_result']

def _get_applicazione_sottotitolo_from():
    """
    get a dict {'slug': 'sottotitolo_from'} from active Applicazione objects
    """
    return {app.codice: app.sottotitolo_from for app in Applicazione.active_objects.all()}

def _is_session_empty(request):
    key_list = [
        'app_posizioni_intestatario', 'app_posizioni_cointestatario',
        'app_posizioni_to', 'app_dati_servizio', 'app_result'
    ]
    for key in key_list:
        if key in request.session:
            return False
    return True


class ServizioFromPlugin(CMSPluginBase):
    model = ServizioFromSettings
    name = _("%s - Posizione From") % ServizioFromSettings._meta.app_label
    module = _("Acomea App")
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "cms_plugins/empty.html"

        if user.is_anonymous() or not instance.status:
            return context

        if not user.is_active:
            return context

        try:
            profilo = user.profilo
            rac = user.profilo.rac
        except:
            return context

        if not profilo.dispositivo and not profilo.is_acomea:
            return context

        self.render_template = instance.form_template_name

        # controllo dati_servizio in sessione,
        id_servizio = None
        if 'app_dati_servizio' in request.session:
            if 'id' in request.session['app_dati_servizio']:
                id_servizio = request.session['app_dati_servizio']['id']

        servizi = profilo.servizi.filter(id=id_servizio).exclude(status="FN").order_by('-id')
        if servizi:
            servizio = servizi[0]
            context.update({
                'object': instance,
                'servizio': servizio,
                'placeholder': placeholder,
            })
            return context

        form_class = import_class(instance.form)
        servizio = Servizio()
        op_servizi = OpServizi()
        classe_fondi = None
        try:
            cliente = op_servizi.get_cliente(rac.codice_fiscale)
            classe_fondi = cliente.get("ClasseFondi", None)
        except Exception, e:
            pass

        # modifico i dati dell'operazione se l'utente è AcomeA e vuole simulare il cliente
        if profilo.is_acomea and 'codice_fiscale' in request.session:
            codice_fiscale = request.session['codice_fiscale']
            servizio.set_cliente_from_codice_fiscale(profilo=profilo, codice_fiscale=codice_fiscale)
            servizio.simulazione = True
        else:
            servizio.set_cliente_from_profile(profilo)
            servizio.simulazione = False

        if request.method == 'POST' and 'app_posizione_from_form_submit' in request.POST:
            posizione_from_form = form_class(request.POST, session=request.session)
            posizioni_intestatario = request.session['app_posizioni_intestatario']
            posizioni_cointestatario = request.session['app_posizioni_cointestatario']

            if posizione_from_form.is_valid():
                # intestatario
                if posizione_from_form.cleaned_data.get('posizione_from_int'):
                    servizio.set_intestatario_position_from(
                        int(posizione_from_form.cleaned_data['posizione_from_int']),
                        posizione_from_form.cleaned_data['tipo_operazione']
                    )

                # cointestatario
                if posizione_from_form.cleaned_data.get('posizione_from_coint'):
                    servizio.set_cointestatario_position_from(
                        int(posizione_from_form.cleaned_data['posizione_from_coint']),
                        posizione_from_form.cleaned_data['tipo_operazione']
                    )

                servizio.save()
                # aggiorno sessione
                dati_servizio = Servizio.objects.filter(id=servizio.id).values()[0]
                request.session['app_dati_servizio'] = dati_servizio
        else:
            _reset_position_from_dict(request)
            dati_servizio = {}
            posizioni_intestatario = servizio.get_intestatario_positions_from()
            posizioni_cointestatario = servizio.get_cointestatario_positions_from()
            request.session['app_posizioni_intestatario'] = posizioni_intestatario
            request.session['app_posizioni_cointestatario'] = posizioni_cointestatario
            request.session['app_dati_servizio'] = dati_servizio
            posizione_from_form = form_class(session=request.session)


        context.update({
            'object': instance,
            'servizio': servizio,
            'classe_fondi': classe_fondi,
            'posizioni_intestatario': posizioni_intestatario,
            'posizioni_cointestatario': posizioni_cointestatario,
            'posizione_from_form': posizione_from_form,
            'placeholder': placeholder,
            'app_sottotitolo_from': _get_applicazione_sottotitolo_from()
        })
        return context
plugin_pool.register_plugin(ServizioFromPlugin)


class ServizioMultiFromPlugin(CMSPluginBase):
    model = ServizioFromSettings
    name = _("%s - Posizione Multi From") % ServizioFromSettings._meta.app_label
    module = _("Acomea App")
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "cms_plugins/empty.html"

        if user.is_anonymous() or not instance.status:
            return context

        if not user.is_active:
            return context

        try:
            profilo = user.profilo
            rac = user.profilo.rac
        except:
            return context

        if not profilo.dispositivo and not profilo.is_acomea:
            return context

        self.render_template = instance.form_template_name

        # controllo dati_servizio in sessione,
        id_servizio = None
        if 'app_dati_servizio' in request.session:
            if 'id' in request.session['app_dati_servizio']:
                id_servizio = request.session['app_dati_servizio']['id']

        servizi = profilo.servizi.filter(id=id_servizio).exclude(status="FN").order_by('-id')
        if servizi:
            servizio = servizi[0]
            context.update({
                'object': instance,
                'servizio': servizio,
                'placeholder': placeholder,
            })
            return context

        form_class = import_class(instance.form)
        servizio = Servizio()

        op_servizi = OpServizi()
        classe_fondi = None
        try:
            cliente = op_servizi.get_cliente(rac.codice_fiscale)
            classe_fondi = cliente.get("ClasseFondi", None)
        except Exception, e:
            pass

        # modifico i dati dell'operazione se l'utente è AcomeA e vuole simulare il cliente
        if profilo.is_acomea and 'codice_fiscale' in request.session:
            codice_fiscale = request.session['codice_fiscale']
            servizio.set_cliente_from_codice_fiscale(profilo=profilo, codice_fiscale=codice_fiscale)
            servizio.simulazione = True
        else:
            servizio.set_cliente_from_profile(profilo)
            servizio.simulazione = False

        if request.method == 'POST' and 'app_posizione_from_form_submit' in request.POST:
            posizione_from_form = form_class(request.POST, session=request.session)
            posizioni_intestatario = request.session['app_posizioni_intestatario']
            posizioni_cointestatario = request.session['app_posizioni_cointestatario']

            if posizione_from_form.is_valid():
                servizio.tipo_operazione = posizione_from_form.cleaned_data['tipo_operazione']
                if servizio.tipo_operazione != "CARDIE":
                    # intestatario
                    if posizione_from_form.cleaned_data.get('posizione_from_int'):
                        servizio.set_intestatario_position_from(
                            int(posizione_from_form.cleaned_data['posizione_from_int'][0]),
                            posizione_from_form.cleaned_data['tipo_operazione']
                        )

                    # cointestatario
                    if posizione_from_form.cleaned_data.get('posizione_from_coint'):
                        servizio.set_cointestatario_position_from(
                            int(posizione_from_form.cleaned_data['posizione_from_coint'][0]),
                            posizione_from_form.cleaned_data['tipo_operazione']
                        )
                    servizio.save()

                ##############
                # CARPE DIEM #
                ##############
                if servizio.tipo_operazione == "CARDIE":
                    # posizioni_from_formset = PosizioneFromFormSet(request.POST, instance=servizio)
                    # if posizioni_from_formset.is_valid():
                    #    pass
                    servizio.posizioni_from.all().delete()
                    posizione_from_int = posizione_from_form.cleaned_data['posizione_from_int']
                    posizione_from_coint = posizione_from_form.cleaned_data['posizione_from_coint']
                    if posizione_from_int:
                        servizio = servizio.set_multi_from(
                            servizio,
                            posizioni_intestatario,
                            posizione_from_int
                        )

                    if posizione_from_coint:
                        servizio = servizio.set_multi_from(
                            servizio,
                            posizioni_cointestatario,
                            posizione_from_coint
                        )

                ###########################
                # RIBILANCIA INVESTIMENTO #
                ###########################
                if servizio.tipo_operazione == "RIBINV":
                    servizio.posizioni_from.all().delete()
                    posizione_from_int = posizione_from_form.cleaned_data['posizione_from_int']
                    posizione_from_coint = posizione_from_form.cleaned_data['posizione_from_coint']
                    if posizione_from_int:
                        servizio = servizio.set_multi_from(
                            servizio,
                            posizioni_intestatario,
                            posizione_from_int
                        )

                    if posizione_from_coint:
                        servizio = servizio.set_multi_from(
                            servizio,
                            posizioni_cointestatario,
                            posizione_from_coint
                        )

                # aggiorno sessione
                dati_servizio = Servizio.objects.filter(id=servizio.id).values()[0]
                request.session['app_dati_servizio'] = dati_servizio
        else:
            _reset_position_from_dict(request)
            dati_servizio = {}
            posizioni_intestatario = servizio.get_intestatario_positions_from()
            posizioni_cointestatario = servizio.get_cointestatario_positions_from()
            request.session['app_posizioni_intestatario'] = posizioni_intestatario
            request.session['app_posizioni_cointestatario'] = posizioni_cointestatario
            request.session['app_dati_servizio'] = dati_servizio

            posizione_from_form = form_class(session=request.session)

        context.update({
            'object': instance,
            'servizio': servizio,
            'classe_fondi': classe_fondi,
            'posizioni_intestatario': posizioni_intestatario,
            'posizioni_cointestatario': posizioni_cointestatario,
            'posizione_from_form': posizione_from_form,
            'placeholder': placeholder,
            'app_sottotitolo_from': _get_applicazione_sottotitolo_from()
        })
        return context
plugin_pool.register_plugin(ServizioMultiFromPlugin)


class ServizioToPlugin(CMSPluginBase):
    model = ServizioToSettings
    name = _("%s - Posizione To") % ServizioToSettings._meta.app_label
    module = _("Acomea App")
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user

        self.render_template = "cms_plugins/empty.html"

        if user.is_anonymous() or not instance.status:
            return context

        if not user.is_active:
            return context

        try:
            profilo = user.profilo
            rac = user.profilo.rac
        except:
            return context

        if not profilo.dispositivo and not profilo.is_acomea:
            return context

        id_servizio = None
        if 'app_dati_servizio' in request.session:
            if 'id' in request.session['app_dati_servizio']:
                id_servizio = request.session['app_dati_servizio']['id']

        servizi = profilo.servizi.filter(status="IN", id=id_servizio).order_by('-id')

        posizioni_to_ex = []
        posizioni_to_not = []
        posizioni_to = []
        posizioni_pianis_rid = []
        posizioni_pianis_swipro = []
        fondi_to_totali = []

        if not servizi:
            return context

        servizio = servizi[0]
        error_message = None

        try:
            applicazione = Applicazione.active_objects.get(codice=servizio.tipo_operazione)
        except:
            return context

        self.render_template = applicazione.to_form_template_name
        form_class = import_class(applicazione.to_form)

        # reset posizione
        if request.method == 'POST' and 'app_posizione_to_form_reset' in request.POST:
            servizio.reset_position_from()
            _reset_position_from_dict(request)
            raise ForceResponse(HttpResponseRedirect('/my_acomea/app/'))

        if request.method == 'POST' and 'app_posizione_to_form_submit' in request.POST:
            posizione_to_form = form_class(request.POST, session=request.session)
            fondi_to_totali = request.session['app_fondi_to_totali']

            if posizione_to_form.is_valid():
                tipo_operazione = servizio.tipo_operazione

                #################
                # ATTIVA CEDOLA #
                #################
                if tipo_operazione == "ATTCED":
                    position_to = posizione_to_form.cleaned_data['posizione_to']
                    # scelta destinazione fondo per la riallocazione
                    if position_to:
                        servizio.data_decorrenza = datetime.date.today().strftime('%Y-%m-%d')
                        position = servizio.set_position_to(int(position_to))
                    # scelta rimborso su C/C
                    else:
                        servizio.set_empty_rapp_position_to()

                ##############
                # CARPE DIEM #
                ##############
                elif tipo_operazione == "CARDIE":
                    servizio.set_position_to()
                    # dati aggiuntivi
                    if 'car_die_gruppo_a' in posizione_to_form.cleaned_data:
                        funds_data_a = posizione_to_form.funds_data_a.get(
                            posizione_to_form.cleaned_data.get('car_die_gruppo_a')
                        )
                        servizio.car_die_gruppo_a_desc = funds_data_a.get('DESCRFON')
                    if 'car_die_gruppo_b' in posizione_to_form.cleaned_data:
                        funds_data_b = posizione_to_form.funds_data_b.get(
                            posizione_to_form.cleaned_data.get('car_die_gruppo_b')
                        )
                        servizio.car_die_gruppo_b_desc = funds_data_b.get('DESCRFON')

                #####################
                # RADDOPPIA RIBASSO #
                #####################
                elif tipo_operazione == "RADRIB":
                    servizio.data_decorrenza = datetime.date.today().strftime('%Y-%m-%d')
                    if posizione_to_form.cleaned_data['posizioni_pianis_rid']:
                        servizio.rad_rib_nop_pianis = int(posizione_to_form.cleaned_data['posizioni_pianis_rid'])
                        servizio.set_position_to_radrib(servizio.rad_rib_nop_pianis)
                    elif posizione_to_form.cleaned_data['posizioni_pianis_swipro']:
                        servizio.rad_rib_nop_pianis = int(posizione_to_form.cleaned_data['posizioni_pianis_swipro'])
                        servizio.set_position_to_radrib(servizio.rad_rib_nop_pianis)

                ######################
                # REINVESTI PROFITTI #
                ######################
                elif tipo_operazione == "REIPRO":
                    servizio.data_decorrenza = datetime.date.today().strftime('%Y-%m-%d')
                    position_to_ex = posizione_to_form.cleaned_data['posizione_to_ex']
                    position_to_not = posizione_to_form.cleaned_data['posizione_to_not']
                    if position_to_ex:
                        servizio.set_position_to(int(position_to_ex), 'ex')
                    elif position_to_not:
                        servizio.set_position_to(int(position_to_not), 'not')

                ###########################
                # RIBILANCIA INVESTIMENTO #
                ###########################
                elif tipo_operazione == "RIBINV":
                    servizio.set_position_to()
                    servizio.posizioni_to.all().delete()
                    for posizione_to in fondi_to_totali:
                        cod_fondo = "fondo_{}_{}_{}".format(
                            posizione_to['CODFONDO'], posizione_to['RAPPORTO'], posizione_to['NUMCONTRATTO']
                        )
                        posizione_to['PESO'] = posizione_to_form.cleaned_data[cod_fondo]
                        if posizione_to['PESO'] > 0:
                            posizioni_to.append(posizione_to)
                    servizio.set_multi_to(servizio, posizioni_to)

                ########################
                # RIMBORSO PROGRAMMATO #
                ########################
                elif tipo_operazione == "RIMPRO":
                    servizio.set_position_to()
                    importo = posizione_to_form.cleaned_data['importo']
                    quote = posizione_to_form.cleaned_data['quote']

                ######################
                # SWITCH PROGRAMMATO #
                ######################
                elif tipo_operazione == "SWIPRO":
                    position_to_ex = posizione_to_form.cleaned_data['posizione_to_ex']
                    position_to_not = posizione_to_form.cleaned_data['posizione_to_not']

                    if position_to_ex:
                        position = servizio.set_position_to(int(position_to_ex), 'ex')
                    elif position_to_not:
                        position = servizio.set_position_to(int(position_to_not), 'not')

                    numero_rate_versate = None

                    if posizione_to_form.cleaned_data['numero_rate_versate']:
                        importo = posizione_to_form.cleaned_data['importo']
                        numero_rate_versate = posizione_to_form.cleaned_data['numero_rate_versate']
                    else:
                        importo = posizione_to_form.cleaned_data['importo']

                else:
                    return context

                # mifid checkato, imposto operazione.mifid a True
                if 'mifid' in request.session['app_dati_servizio']:
                    if request.session['app_dati_servizio']['mifid']:
                        servizio.mifid = True

                servizio.set_valori(posizione_to_form.cleaned_data)
                servizio.save()

                # aggiorno i dati nella request
                dati_servizio = Servizio.objects.filter(id=servizio.id).values()[0]
                request.session['app_dati_servizio'] = dati_servizio
            else:
                ###########################
                # RIBILANCIA INVESTIMENTO #
                ###########################
                """
                se il form va in errore devono essere ricalcolati i totali
                """
                if servizio.tipo_operazione == "RIBINV":
                    context.update({
                        'ribinv_patrimonio_from_sum': sum([
                            x.get('patrimonio_from') for x in servizio.posizioni_from.values('patrimonio_from')
                        ])
                    })

                ##############
                # CARPE DIEM #
                ##############
                """
                se il form va in errore devono essere ricalcolati i totali
                """
                if servizio.tipo_operazione == "CARDIE":
                    context.update({
                        'cardie_patrimonio_from_sum': sum([
                            x.get('patrimonio_from') for x in servizio.posizioni_from.values('patrimonio_from')
                        ])
                    })

                posizioni_to_ex = request.session['app_posizioni_to_ex']
                posizioni_to_not = request.session['app_posizioni_to_not']
                posizioni_to = request.session['app_posizioni_to']
                posizioni_pianis_rid = request.session['app_posizioni_pianis_rid']
                posizioni_pianis_swipro = request.session['app_posizioni_pianis_swipro']
                request.session['app_result'] = None

        else:
            _reset_position_to_dict(request)

            #################
            # ATTIVA CEDOLA #
            #################
            if servizio.tipo_operazione == "ATTCED":
                posizioni_to = servizio.get_att_ced_positions_to()

            ##############
            # CARPE DIEM #
            ##############
            elif servizio.tipo_operazione == "CARDIE":
                posizioni_to = servizio.get_car_die_position_to()
                request.session['elenco_tcpic'] = servizio.get_car_die_elenco_tcpic()
                context['hide_tcpic'] = True if len(request.session['elenco_tcpic']) == 1 else False
                context['tcpicp_case_num'] = 3
                if len(request.session['elenco_tcpic']) == 1 and not int(request.session['elenco_tcpic'][0]['OpServizi_TCPIC']['NumContratto']):
                    context['tcpicp_case_num'] = 1
                elif len(request.session['elenco_tcpic']) == 1:
                    context['tcpicp_case_num'] = 2

                context.update({
                    'cardie_patrimonio_from_sum': sum([
                        x.get('patrimonio_from') for x in servizio.posizioni_from.values('patrimonio_from')
                    ])
                })
            #####################
            # RADDOPPIA RIBASSO #
            #####################
            elif servizio.tipo_operazione == "RADRIB":
                posizioni_pianis_rid = servizio.get_rad_rib_pianis_positions('RID')
                posizioni_pianis_swipro = servizio.get_rad_rib_pianis_positions('SWIPRO')

            ######################
            # REINVESTI PROFITTI #
            ######################
            elif servizio.tipo_operazione == "REIPRO":
                posizioni_to_ex = servizio.get_swi_pro_ex_positions_to()
                posizioni_to_not = servizio.get_swi_pro_not_positions_to()

            ###########################
            # RIBILANCIA INVESTIMENTO #
            ###########################
            elif servizio.tipo_operazione == "RIBINV":
                servizio.set_servizio()
                fondi_to_totali = servizio.op_servizio.op_servizi_get_elenco_ribinv(
                    servizio.codice_cliente, servizio._get_dettaglio_from()
                )
                context.update({
                    'ribinv_patrimonio_from_sum': sum([
                        x.get('patrimonio_from') for x in servizio.posizioni_from.values('patrimonio_from')
                    ])
                })
            ######################
            # SWITCH PROGRAMMATO #
            ######################
            elif servizio.tipo_operazione == "SWIPRO":
                posizioni_to_ex = servizio.get_swi_pro_ex_positions_to()
                posizioni_to_not = servizio.get_swi_pro_not_positions_to()

            request.session['app_posizioni_to'] = posizioni_to
            request.session['app_posizioni_to_ex'] = posizioni_to_ex
            request.session['app_posizioni_to_not'] = posizioni_to_not
            request.session['app_posizioni_pianis_rid'] = posizioni_pianis_rid
            request.session['app_posizioni_pianis_swipro'] = posizioni_pianis_swipro
            request.session['app_result'] = None
            request.session['app_fondi_to_totali'] = fondi_to_totali

            posizione_to_form = form_class(session=request.session)

        context.update({
            'object': instance,
            'servizio': servizio,
            'posizioni_to_ex': posizioni_to_ex,
            'posizioni_to_not': posizioni_to_not,
            'posizioni_to': posizioni_to,
            'posizioni_pianis_rid': posizioni_pianis_rid,
            'posizioni_pianis_swipro': posizioni_pianis_swipro,
            'posizione_to_form': posizione_to_form,
            'fondi_to_totali': fondi_to_totali,
            'error_message': error_message,
            'placeholder': placeholder,
            'app_sottotitolo_from': _get_applicazione_sottotitolo_from(),
            'app_modal_page_slug': applicazione.modal_page_slug,
            'applicazione': applicazione,
        })
        return context
plugin_pool.register_plugin(ServizioToPlugin)


class ServizioRiepilogoPlugin(CMSPluginBase):
    model = ServizioRiepilogoSettings
    name = _("%s - Riepilogo") % ServizioRiepilogoSettings._meta.app_label
    module = _("Acomea App")
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "cms_plugins/empty.html"
        error_message = None

        if user.is_anonymous() or not instance.status:
            return context

        if not user.is_active:
            return context

        try:
            profilo = user.profilo
            rac = user.profilo.rac
        except:
            return context

        if not profilo.dispositivo and not profilo.is_acomea:
            return context

        # controllo data_operazione in sessione
        id_servizio = None
        if 'app_dati_servizio' in request.session:
            if 'id' in request.session['app_dati_servizio']:
                id_servizio = request.session['app_dati_servizio']['id']

        servizi = Servizio.objects.filter(
            profilo=profilo, status="FF", id=id_servizio
        ).order_by('-id')

        if not servizi:
            return context

        servizio = servizi[0]

        try:
            applicazione = Applicazione.active_objects.get(codice=servizio.tipo_operazione)
        except:
            return context

        self.render_template = applicazione.riepilogo_form_template_name
        form_class = import_class(applicazione.riepilogo_form)

        # reset posizione
        if request.method == 'POST' and 'riepilogo_form_reset' in request.POST:
            servizio.reset_position_to()
            _reset_position_to_dict(request)
            servizio.save()
            #_clean_session(request)
            raise ForceResponse(HttpResponseRedirect('/my_acomea/app/'))

        # set posizione
        if request.method == 'POST' and 'riepilogo_form_submit' in request.POST:
            riepilogo_form = form_class(
                request.POST, 
                session=request.session
            )

            if riepilogo_form.is_valid():
                tipo_operazione = servizio.tipo_operazione
                result = servizio.save_servizio()
                if result['OK'] is True:
                    servizio.save()

        else:
            riepilogo_form = form_class(
                session=request.session
            )

        ##############
        # CARPE DIEM #
        ##############
        if servizio.tipo_operazione == "CARDIE":
            cardie_patrimonio_from_sum = sum([
                x.get('patrimonio_from') for x in 
                    servizio.posizioni_from.values(
                        'patrimonio_from'
                )
            ])
            car_die_allocazione_a_sum = (cardie_patrimonio_from_sum/100)*servizio.car_die_allocazione_a
            car_die_allocazione_b_sum = (cardie_patrimonio_from_sum/100)*servizio.car_die_allocazione_b
            context.update({
                'cardie_patrimonio_from_sum': cardie_patrimonio_from_sum,
                'car_die_allocazione_a_sum': "%.2f" % car_die_allocazione_a_sum,
                'car_die_allocazione_b_sum': "%.2f" % car_die_allocazione_b_sum
            })

        ###########################
        # RIBILANCIA INVESTIMENTO #
        ###########################
        if servizio.tipo_operazione == "RIBINV":
            context.update({
                'ribinv_patrimonio_from_sum': sum([
                    x.get('patrimonio_from') for x in
                    servizio.posizioni_from.values('patrimonio_from')
                ])
            })

        context.update({
            'object': instance,
            'placeholder': placeholder,
            'servizio': servizio,
            'error_message': error_message,
            'riepilogo_form': riepilogo_form,
            'applicazione': applicazione,
        })
        return context
plugin_pool.register_plugin(ServizioRiepilogoPlugin)


class ServiziInseritiPlugin(CMSPluginBase):
    module = _('Acomea App')
    model = ServiziInseritiSettings
    name = _("%s - Servizi Inseriti") % ServizioRiepilogoSettings._meta.app_label
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "cms_plugins/empty.html"

        if user.is_anonymous() or not instance.status:
            return context

        if not user.is_active:
            return context

        try:
            profilo = user.profilo
            rac = user.profilo.rac
        except:
            return context

        if not profilo.is_cliente and not profilo.is_acomea:
            return context

        codice_fiscale = None
        operazioni_internet = None
        operazioni_da_valorizzare = None
        # modifico i dati dell'operazione se l'utente è AcomeA e vuole simulare il cliente
        if profilo.is_acomea and 'codice_fiscale' in request.session:
            codice_fiscale = request.session['codice_fiscale']

        if profilo.is_cliente:
            codice_fiscale = rac.codice_fiscale

        if not codice_fiscale:
            return context

        op_servizi = OpServizi()
        servizi_inseriti = op_servizi.op_servizi_get_servizi_in_corso_web(codice_fiscale)
        servizi_attivi = op_servizi.op_servizi_get_servizi_attivi(codice_fiscale)
        self.render_template = instance.template_name
        context.update({
            'object': instance,
            'servizi_inseriti': servizi_inseriti,
            'servizi_attivi': servizi_attivi,
            'placeholder': placeholder,
        })
        return context
plugin_pool.register_plugin(ServiziInseritiPlugin)
