#-*- coding: utf-8 -*-

import datetime
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

from jmb.core.db.models import DateModel, UserModel, StatusModel

from acomea.app.soap import OpServizi
from acomea.profilo.models import AcomeaProfilo

MESI = (
    (0, 'Mese'),
    (1, 'Gennaio',), (2, 'Febbraio',), (3, 'Marzo',),
    (4, 'Aprile'), (5, 'Maggio',), (6, 'Giugno',),
    (7, 'Luglio',), (8, 'Agosto'), (9, 'Settembre',),
    (10, 'Ottobre',), (11, 'Novembre',), (12, 'Dicembre')
)


class ApplicazioneManager(models.Manager):
    def get_query_set(self):
        return super(ApplicazioneManager, self).get_query_set()


class ActiveApplicazioneManager(models.Manager):
    def get_query_set(self):
        qs = super(ActiveApplicazioneManager, self).get_query_set()
        return qs.filter(status=1)


class Applicazione(UserModel, DateModel, StatusModel):
    objects = ApplicazioneManager()
    active_objects = ActiveApplicazioneManager()

    codice = models.CharField(
        max_length=6,
        unique=True,
        choices=settings.APP_SERVIZIO_TIPO_SERVIZIO_CHOICES,
    )
    modal_page_slug = models.SlugField(
        _("Modal page slug"),
        max_length=100,
        null=True, blank=True,
        help_text=_("(Optional) url for remote modal info page")
    )
    description = models.CharField(
        _("Description"),
        max_length=250,
    )
    sottotitolo_from = models.CharField(
        max_length=100,
        null=True, blank=True,
        help_text=_("(Optional) per inserire sottotitolo nella pagina APP From")
    )
    from_form = models.CharField(
        _("POSIZIONE FROM Form"),
        max_length=100,
        null=True, blank=True,
        choices=settings.APP_SERVIZIO_CMSPLUGIN_POSIZIONEFROM_FORM_CLASSES,
    )
    # from_form_template_name = models.CharField(
    #     _("POSIZIONE FROM Form template"),
    #     max_length=100,
    #     choices=settings.APP_SERVIZIO_CMSPLUGIN_POSIZIONEFROM_FORM_TEMPLATES,
    # )
    to_form = models.CharField(
        _("POSIZIONE TO Form"),
        max_length=100,
        choices=settings.APP_SERVIZIO_CMSPLUGIN_POSIZIONETO_FORM_CLASSES,
    )
    to_form_template_name = models.CharField(
        _("POSIZIONE TO Form template"),
        max_length=100,
        choices=settings.APP_SERVIZIO_CMSPLUGIN_POSIZIONETO_FORM_TEMPLATES,
    )
    riepilogo_form = models.CharField(
        _("POSIZIONE RIEPILOGO Form"),
        max_length=100,
        choices=settings.APP_SERVIZIO_CMSPLUGIN_RIEPILOGO_FORM_CLASSES,
    )
    riepilogo_form_template_name = models.CharField(
        _("POSIZIONE RIEPILOGO template"),
        max_length=100,
        choices=settings.APP_SERVIZIO_CMSPLUGIN_RIEPILOGO_FORM_TEMPLATES,
    )
    costo_applicazione = models.DecimalField(
        _("Costo Applicazione"),
        default=0, decimal_places=2, max_digits=10
    )
    messaggio_gratuito = models.TextField(
        _("Messaggio Applicazione Gratuita"),
    )
    messaggio_pagamento = models.TextField(
        _("Messaggio Applicazione a Pagamento"),
    )
    data_inizio_pagamento = models.DateField(
        _("Data Inizio a Pagamento"),
        #default=datetime.date.today,
    )

    class Meta(object):
        verbose_name = _("Applicazione")
        verbose_name_plural = _("Applicazioni")

    def __unicode__(self):
        return 'Applicazione: %s' % self.get_codice_display()

    def get_pagamento_label(self, check_date=None):
        check_date = datetime.date.today() if not check_date else check_date
        if check_date >= self.data_inizio_pagamento and self.costo_applicazione > 0:
            return self.messaggio_pagamento
        return self.messaggio_gratuito


class PosizioneFrom(UserModel, DateModel):
    servizio = models.ForeignKey('Servizio', related_name='posizioni_from')
    codice_rapporto_from = models.IntegerField(blank=True, null=True)
    codice_deposito_from = models.IntegerField(blank=True, null=True)
    codice_fondo_from = models.CharField(max_length=5, blank=True, null=True)
    descrizione_fondo_from = models.CharField(max_length=255, blank=True, null=True)
    codice_tipocontratto_from = models.CharField(max_length=5, blank=True, null=True)
    tipo_piano_from = models.CharField(max_length=5, blank=True, null=True)
    patrimonio_from = models.DecimalField(default=0, decimal_places=2, max_digits=10)
    quote_from = models.DecimalField(default=0, decimal_places=3, max_digits=10)

    class Meta(object):
        verbose_name = _("Posizione From")
        verbose_name_plural = _("Posizioni From")

    def __unicode__(self):
        return 'Posizione From: ' + str(self.__dict__)


class PosizioneTo(UserModel, DateModel):
    servizio = models.ForeignKey('Servizio', related_name='posizioni_to')
    codice_rapporto_to = models.IntegerField(blank=True, null=True)
    codice_deposito_to = models.IntegerField(blank=True, null=True)
    codice_fondo_to = models.CharField(max_length=5, blank=True, null=True)
    descrizione_fondo_to = models.CharField(max_length=255, blank=True, null=True)
    codice_tipocontratto_to = models.CharField(max_length=5, blank=True, null=True)
    tipo_piano_to = models.CharField(max_length=5, blank=True, null=True)
    peso_to = models.DecimalField(max_digits=4, decimal_places=2)

    class Meta(object):
        verbose_name = _("Posizione To")
        verbose_name_plural = _("Posizioni To")

    def __unicode__(self):
        return 'Posizione To: ' + str(self.__dict__)


class Servizio(UserModel, DateModel):
    status = models.CharField(
        max_length=3,
        choices=settings.APP_SERVIZIO_STATUS_CHOICES
    )
    profilo = models.ForeignKey(
        AcomeaProfilo,
        related_name="servizi",
        verbose_name=_("Profilo")
    )
    codice_cliente = models.IntegerField()
    codice_fiscale = models.CharField(max_length=16)

    tipo_operazione = models.CharField(
        max_length=7,
        choices=settings.APP_SERVIZIO_TIPO_SERVIZIO_CHOICES,
    )

    codice_rapporto_from = models.IntegerField(blank=True, null=True)
    codice_deposito_from = models.IntegerField(blank=True, null=True)
    codice_fondo_from = models.CharField(max_length=5, blank=True, null=True)
    descrizione_fondo_from = models.CharField(max_length=255, blank=True, null=True)
    codice_tipocontratto_from = models.CharField(max_length=5, blank=True, null=True)
    tipo_piano_from = models.CharField(max_length=5, blank=True, null=True)
    patrimonio_from = models.DecimalField(default=0, decimal_places=2, max_digits=10)
    quote_from = models.DecimalField(default=0, decimal_places=3, max_digits=10)

    codice_rapporto_to = models.IntegerField(blank=True, null=True)
    codice_deposito_to = models.IntegerField(blank=True, null=True)
    codice_fondo_to = models.CharField(max_length=5, blank=True, null=True)
    descrizione_fondo_to = models.CharField(max_length=255, blank=True, null=True)
    codice_tipocontratto_to = models.CharField(max_length=5, blank=True, null=True)
    tipo_piano_to = models.CharField(max_length=5, blank=True, null=True)

    tipo_qta_movimento = models.CharField(
        max_length=1,
        choices=settings.APP_SERVIZIO_TIPO_QTA_MOVIMENTO_CHOICES,
        blank=True, null=True
    )

    cognome = models.CharField(max_length=50, blank=True, null=True)
    nome = models.CharField(max_length=50, blank=True, null=True)
    mifid = models.BooleanField(default=False)
    importo = models.DecimalField(default=0, decimal_places=2, max_digits=10)
    numero_rate_piano = models.IntegerField(blank=True, null=True)
    importo_rata = models.DecimalField(default=0, decimal_places=2, max_digits=10)
    numero_rate_versate = models.DecimalField(blank=True, null=True, decimal_places=2, max_digits=10)
    quote = models.DecimalField(blank=True, null=True, decimal_places=3, max_digits=10)  # x switch o rimborso
    iban = models.CharField(max_length=27, blank=True, null=True)

    simulazione = models.BooleanField(default=True)  # utenti AvomeA sempre in 'simulazione'
    numero_operazione = models.IntegerField(blank=True, null=True)
    data_operazione = models.DateTimeField(blank=True, null=True)

    frequenza = models.CharField(
        max_length=1,
        null=True, blank=True,
        choices=settings.APP_SERVIZIO_FREQUENZA_CHOICES,
    )
    data_decorrenza = models.DateField(
        null=True, blank=True,
    )
    fine_servizio = models.CharField(
        max_length=1,
        null=True, blank=True,
        choices=settings.APP_SERVIZIO_FINE_SERVIZIO_CHOICES,
    )
    fine_servizio_anno = models.PositiveSmallIntegerField(
        null=True, blank=True,
    )
    fine_servizio_mese = models.PositiveSmallIntegerField(
        choices=MESI,
        null=True, blank=True,
    )

    rei_pro_perc_plus = models.PositiveSmallIntegerField(
        default=0,
        verbose_name="Perc Plus %"
    )

    car_die_gruppo_a = models.CharField(
        max_length=2,
        default='',
        verbose_name="Fondo Gruppo A"
    )
    car_die_gruppo_b = models.CharField(
        max_length=2,
        default='',
        verbose_name="Fondo Gruppo B"
    )
    car_die_allocazione_a = models.PositiveSmallIntegerField(
        default=0,
        verbose_name="Allocazione A %"
    )
    car_die_allocazione_b = models.PositiveSmallIntegerField(
        default=0,
        verbose_name="Allocazione B %"
    )
    car_die_perc_var_a = models.PositiveSmallIntegerField(
        default=0,
        verbose_name="Perc Variazione A %"
    )
    car_die_perc_riall_ab = models.PositiveSmallIntegerField(
        default=0,
        verbose_name="Perc Riallineamento AB %"
    )
    rib_inv_perc_scost = models.PositiveSmallIntegerField(
        default=0,
        verbose_name="Perc di Scostamento %"
    )
    rad_rib_flag = models.BooleanField(
        default=False,
        verbose_name="Flag Raddoppia"
    )
    rad_rib_riduz_quota_a = models.PositiveSmallIntegerField(
        default=0,
        verbose_name="Riduzione quota A %"
    )
    rad_rib_riduz_quota_b = models.PositiveSmallIntegerField(
        default=0,
        verbose_name="Riduzione quota B %"
    )
    rad_rib_nop_pianis = models.IntegerField(
        null=True, blank=True,
        verbose_name="N. Servizio"
    )
    car_die_gruppo_a_desc = models.CharField(
        null=True, blank=True,
        max_length=50,
        verbose_name="Descrizione Gruppo A"
    )
    car_die_gruppo_b_desc = models.CharField(
        null=True, blank=True,
        max_length=50,
        verbose_name="Descrizione Gruppo B"
    )
    messaggio_costo_applicazione = models.TextField(
        _("Messaggio costo applicazione"),
    )
    costo_applicazione = models.DecimalField(
        _("Costo Applicazione"),
        default=0, decimal_places=2, max_digits=10
    )

    def set_servizio(self):
        os = OpServizi()
        self.op_servizio = os

    def set_cliente(self):
        if not hasattr(self, 'op_servizio'):
            self.set_servizio()
        self.cliente = self.op_servizio.get_cliente(self.codice_fiscale)

    def set_cliente_from_profile(self, profilo):
        try:
            self.codice_fiscale = profilo.rac.codice_fiscale
            self.profilo = profilo
            self.set_codice_cliente()
        except:
            pass

    def set_cliente_from_codice_fiscale(self, profilo, codice_fiscale):
        try:
            self.codice_fiscale = codice_fiscale
            self.profilo = profilo
            self.set_codice_cliente()
        except:
            pass

    def set_codice_cliente(self):
        if not hasattr(self, 'cliente'):
            self.set_cliente()
        self.codice_cliente = self.cliente['CodCliFC']

    def is_area_active(self):
        if not hasattr(self, 'op_servizio'):
            self.set_servizio()
        return self.op_servizio.is_area_active()

    def is_persona_fisica(self):
        if not hasattr(self, 'cliente'):
            self.set_cliente()
        if self.cliente['TipoPartecipante'].strip() == "1":
            return True
        return False

    def get_cliente(self):
        if not hasattr(self, 'cliente'):
            self.set_cliente()
        return self.cliente

    def get_codice_cliente(self):
        if not self.codice_cliente:
            self.set_codice_cliente()
        return self.codice_cliente

    def is_intestatario(self):
        if not hasattr(self, 'cliente'):
            self.set_cliente()
        return self.cliente['IsIntestatario']

    def is_cointestatario(self):
        if not hasattr(self, 'cliente'):
            self.set_cliente()
        return self.cliente['IsCoIntestatario']

    def get_intestatario_positions_from(self):
        """
        restituisce le posizioni intestatario
        """
        if not hasattr(self, 'op_servizio'):
            self.set_servizio()
        return self.op_servizio.op_servizi_datagrid(self.codice_cliente, 'INTESTATARIO')

    def get_cointestatario_positions_from(self):
        """
        restituisce le posizioni intestatario
        """
        if not hasattr(self, 'op_servizio'):
            self.set_servizio()
        return self.op_servizio.op_servizi_datagrid(self.codice_cliente, 'COINTESTATARIO')

    def set_intestatario_position_from(self, index, tipo_operazione):
        try:
            position = self.get_intestatario_positions_from()[index]
            self.status = 'IN'
            self.codice_rapporto_from = position['RAPPORTO']
            self.codice_deposito_from = position['NUMCONTRATTO']
            self.codice_fondo_from = position['CODFONDO']
            self.codice_tipocontratto_from = position['CODTC']
            self.descrizione_fondo_from = position['DESCRFONDO']
            self.tipo_piano_from = position['TIPOPIANO']
            self.patrimonio_from = position['PATRIMONIO']
            self.quote_from = position['QUOTE']
            self.tipo_operazione = tipo_operazione
            return position
        except Exception, e:
            return None

    def set_cointestatario_position_from(self, index, tipo_operazione):
        try:
            position = self.get_cointestatario_positions_from()[index]
            self.status = "IN"
            self.codice_rapporto_from = position['RAPPORTO']
            self.codice_deposito_from = position['NUMCONTRATTO']
            self.codice_fondo_from = position['CODFONDO']
            self.codice_tipocontratto_from = position['CODTC']
            self.descrizione_fondo_from = position['DESCRFONDO']
            self.tipo_piano_from = position['TIPOPIANO']
            self.patrimonio_from = position['PATRIMONIO']
            self.quote_from = position['QUOTE']
            self.tipo_operazione = tipo_operazione
            return position
        except Exception, e:
            return None

    def set_multi_from(self, servizio, posizioni, posizione_from):
        """
        metodo per creazione posizioni_to per CARDIE e RIBINV INTESTATARIO e COINTESTATARIO
        """
        servizio.status = 'IN'
        servizio.tipo_operazione = servizio.tipo_operazione
        servizio.codice_rapporto_from = 0
        servizio.codice_deposito_from = 0
        servizio.codice_fondo_from = ''
        servizio.codice_tipocontratto_from = ''
        servizio.save()
        for ind in posizione_from:
            posizione_scelta = posizioni[int(ind)]
            posizione = PosizioneFrom()
            posizione.servizio = servizio
            posizione.codice_rapporto_from = int(posizione_scelta['RAPPORTO'])
            posizione.codice_deposito_from = int(posizione_scelta['NUMCONTRATTO'])
            posizione.codice_fondo_from = posizione_scelta['CODFONDO']
            posizione.descrizione_fondo_from = posizione_scelta['DESCRFONDO']
            posizione.codice_tipocontratto_from = posizione_scelta['CODTC']
            posizione.tipo_piano_from = posizione_scelta['TIPOPIANO']
            posizione.patrimonio_from = posizione_scelta['PATRIMONIO']
            posizione.quote_from = posizione_scelta['QUOTE']
            posizione.save()
        return servizio

    def set_multi_to(self, servizio, posizioni_to):
        """
        metodo per creazione posizioni_to per RIBINV
        """
        servizio.status = "FF"
        servizio.save()
        for posizione_to in posizioni_to:
            posizione = PosizioneTo()
            posizione.servizio = servizio
            posizione.codice_rapporto_to = posizione_to['RAPPORTO']
            posizione.codice_deposito_to = posizione_to['NUMCONTRATTO']
            posizione.codice_fondo_to = posizione_to['CODFONDO']
            posizione.descrizione_fondo_to = posizione_to['DESCRFONDO']
            posizione.codice_tipocontratto_to = posizione_to['CODTC']
            posizione.tipo_piano_to = posizione_to['TIPOPIANO']
            posizione.peso_to = posizione_to['PESO']
            posizione.save()

    def reset_position_from(self):
        self.delete()

    def set_valori(self, dati_operazione_dict):

        ## ATTCED
        if self.tipo_operazione == "ATTCED":
            self.fine_servizio = 'R'
            self.__dict__.update(dati_operazione_dict)

        ## CARDIE
        elif self.tipo_operazione == "CARDIE":
            self.fine_servizio = 'R'
            self.codice_rapporto_to = 0
            self.codice_deposito_to = 0
            self.codice_fondo_to = ''
            self.codice_tipocontratto_to = ''
            self.__dict__.update(dati_operazione_dict)
            if dati_operazione_dict.get("tcpic", None) and dati_operazione_dict["tcpic"].isdigit():
                self.codice_deposito_to = int(dati_operazione_dict["tcpic"])

        ## RADRIB
        elif self.tipo_operazione == "RADRIB":
            self.__dict__.update(dati_operazione_dict)

            if self.fine_servizio == 'R':
                self.fine_servizio_anno = 0
                self.fine_servizio_mese = 0

        ## REIPRO
        elif self.tipo_operazione == "REIPRO":
            self.__dict__.update(dati_operazione_dict)

            if self.fine_servizio == 'R':
                self.fine_servizio_anno = 0
                self.fine_servizio_mese = 0

        ## CARDIE
        elif self.tipo_operazione == "RIBINV":
            self.fine_servizio = 'R'
            self.codice_rapporto_to = 0
            self.codice_deposito_to = 0
            self.codice_fondo_to = ''
            self.codice_tipocontratto_to = ''
            self.__dict__.update(dati_operazione_dict)

        ## RIMPRO
        elif self.tipo_operazione == "RIMPRO":
            self.__dict__.update(dati_operazione_dict)

            if self.tipo_piano_to == 'PIC':
                self.numero_rate_versate = 0
                self.numero_rate_piano = 0
                self.importo_rata = 0

                if self.tipo_qta_movimento == 'I' and self.importo > 0:
                    self.quote = 0
                elif self.tipo_qta_movimento == 'Q' and self.quote > 0:
                    self.importo = 0
                else:
                    return "ERRORE 1"  # FIXME: errore

            if self.tipo_piano_to in ('PAC', 'PAD'):
                if self.tipo_qta_movimento == 'I' and self.importo > 0:
                    self.quote = 0
                elif self.tipo_qta_movimento == 'Q' and self.quote > 0:
                    self.importo = 0
                else:
                    return "ERRORE 1"  # FIXME: errore

            if self.fine_servizio == 'R':
                self.fine_servizio_anno = 0
                self.fine_servizio_mese = 0

        ## SWIPRO ##
        elif self.tipo_operazione == "SWIPRO":
            self.__dict__.update(dati_operazione_dict)

            if self.tipo_piano_to == 'PIC':
                self.numero_rate_versate = 0
                self.numero_rate_piano = 0
                self.importo_rata = 0

                if (
                    self.tipo_qta_movimento == 'I' and self.importo > 0
                    and not self.numero_rate_versate and not self.quote
                ):
                    self.quote = 0
                elif self.tipo_qta_movimento == 'Q' and self.quote > 0 and not self.numero_rate_versate:
                    self.importo = 0
                else:
                    return "ERRORE 1"  # FIXME: errore

            if self.tipo_piano_to in ('PAC', 'PAD'):
                if self.tipo_qta_movimento == 'I':
                    self.quote = 0
                else:
                    return "ERRORE 1"  # FIXME: errore

            if self.fine_servizio == 'R':
                self.fine_servizio_anno = 0
                self.fine_servizio_mese = 0

        # else finale se tipo_operazione non è tra quelle definite
        else:
            return "ERRORE 2"  # FIXME: errore

        self.status = 'FF'

    def _get_dettaglio_from(self):
        """
        '<rapporto1>';'<contratto1>';<'tipocontratto1'>;'<codice_fondo1>';'<patrimonio1>'|
        '<rapporto2>';'<contratto2>';<'tipocontratto2'>;'<codice_fondo2>';'<patrimonio2>'|
        ...
        '<rapportoN>';'<contrattoN>';<'tipocontrattoN'>;'<codice_fondoN>';'<patrimonioN>'
        """
        lista = []

        for posizione_from in self.posizioni_from.all():
            lista.append("%s;%s;%s;%s;%s" % (
                posizione_from.codice_rapporto_from, posizione_from.codice_deposito_from,
                posizione_from.codice_tipocontratto_from, posizione_from.codice_fondo_from,
                posizione_from.patrimonio_from
            ))
        return "|".join(lista)
    dettaglio_from = property(_get_dettaglio_from)

    def _get_dettaglio_to(self):
        """
        '<rapporto1>';'<contratto1>';<'tipocontratto1'>;'<codice_fondo1>';'<patrimonio1>'|
        '<rapporto2>';'<contratto2>';<'tipocontratto2'>;'<codice_fondo2>';'<patrimonio2>'|
        ...
        '<rapportoN>';'<contrattoN>';<'tipocontrattoN'>;'<codice_fondoN>';'<patrimonioN>'
        """
        lista = []

        for posizione_to in self.posizioni_to.all():
            lista.append("%s;%s;%s;%s;%s" % (
                posizione_to.codice_rapporto_to, posizione_to.codice_deposito_to,
                posizione_to.codice_tipocontratto_to, posizione_to.codice_fondo_to,
                int(posizione_to.peso_to)
            ))
        return "|".join(lista)
    dettaglio_to = property(_get_dettaglio_to)

    #######################
    # ATTIVA CEDOLA       #
    #######################
    def get_att_ced_positions_to(self):
        if not self.tipo_operazione == "ATTCED":
            return None

        if not hasattr(self, 'op_servizio'):
            self.set_servizio()

        return self.op_servizio.op_servizi_get_elenco_switch(
            self.codice_cliente, self.codice_rapporto_from,
            self.codice_deposito_from, self.codice_fondo_from,
            self.codice_tipocontratto_from
        )

    ##############
    # CARPE DIEM #
    ##############
    def get_car_die_position_to(self):
        position = {}
        position['RAPPORTO'] = 0
        position['NUMCONTRATTO'] = 0
        position['CODFONDO'] = ""
        position['DESCRFONDO'] = ""
        position['CODTC'] = ""
        position['TIPOPIANO'] = ""
        position['NUMRATEPAC'] = 0
        position['IMPORTORATA'] = 0
        position['PIANIS'] = 0
        return position

    def get_car_die_elenco_tcpic(self):
        if not self.tipo_operazione == "CARDIE":
            return None

        if not hasattr(self, 'op_servizio'):
            self.set_servizio()

        codice_rapporto_from = self.posizioni_from.all()[0].codice_rapporto_from
        return self.op_servizio.op_servizi_get_elenco_tcpic(self.codice_cliente, codice_rapporto_from)

    #####################
    # RADDOPPIA RIBASSO #
    #####################
    def get_rad_rib_pianis_positions(self, tipo):
        if not self.tipo_operazione == "RADRIB":
            return None

        if not hasattr(self, 'op_servizio'):
            self.set_servizio()

        return self.op_servizio.op_servizi_get_elenco_servizi_pianis(
            tipo, self.codice_rapporto_from
        )

    def get_rad_rib_position_to(self, value):
        position = {}
        position['RAPPORTO'] = self.codice_rapporto_from
        position['NUMCONTRATTO'] = self.codice_deposito_from
        position['CODFONDO'] = self.codice_fondo_from
        position['DESCRFONDO'] = self.descrizione_fondo_from
        position['CODTC'] = self.codice_tipocontratto_from
        position['TIPOPIANO'] = self.tipo_piano_from
        position['NUMRATEPAC'] = 0
        position['IMPORTORATA'] = 0
        position['PIANIS'] = 0
        return position

    #######################
    # REINVESTI PROFITTI  #
    #######################
    def get_rei_pro_positions_to(self):
        if not self.tipo_operazione == "REIPRO":
            return None

        if not hasattr(self, 'op_servizio'):
            self.set_servizio()

        return self.op_servizio.op_servizi_get_elenco_switch(
            self.codice_cliente, self.codice_rapporto_from,
            self.codice_deposito_from, self.codice_fondo_from,
            self.codice_tipocontratto_from
        )

    ############################
    # RIBILANCIA INVESTIMENTI  #
    ############################
    def get_rib_inv_position_to(self):
        position = {}
        position['RAPPORTO'] = 0
        position['NUMCONTRATTO'] = 0
        position['CODFONDO'] = ""
        position['DESCRFONDO'] = ""
        position['CODTC'] = ""
        position['TIPOPIANO'] = ""
        position['NUMRATEPAC'] = 0
        position['IMPORTORATA'] = 0
        position['PIANIS'] = 0
        return position

    ########################
    # RIMBORSO PROGRAMMATO #
    ########################
    def get_rim_pro_position_to(self):
        position = {}
        position['RAPPORTO'] = self.codice_rapporto_from
        position['NUMCONTRATTO'] = self.codice_deposito_from
        position['CODFONDO'] = self.codice_fondo_from
        position['DESCRFONDO'] = self.descrizione_fondo_from
        position['CODTC'] = self.codice_tipocontratto_from
        position['TIPOPIANO'] = self.tipo_piano_from
        position['NUMRATEPAC'] = 0
        position['IMPORTORATA'] = 0
        return position

    #######################
    # SWITCH PROGRAMMATO  #
    #######################
    def get_swi_pro_ex_positions_to(self):
        if not self.tipo_operazione in ["SWIPRO", "REIPRO"]:
            return None

        if not hasattr(self, 'op_servizio'):
            self.set_servizio()

        return self.op_servizio.op_servizi_get_elenco_switch_ex(
            self.codice_cliente, self.codice_rapporto_from,
            self.codice_deposito_from, self.codice_fondo_from,
            self.codice_tipocontratto_from
        )

    def get_swi_pro_not_positions_to(self):
        if not self.tipo_operazione in ["SWIPRO", "REIPRO"]:
            return None

        if not hasattr(self, 'op_servizio'):
            self.set_servizio()

        return self.op_servizio.op_servizi_get_elenco_switch_not(
            self.codice_cliente, self.codice_rapporto_from,
            self.codice_deposito_from, self.codice_fondo_from,
            self.codice_tipocontratto_from
        )

    #######################
    # POSITION TO         #
    #######################
    def set_empty_rapp_position_to(self):
        if not hasattr(self, 'cliente'):
            self.set_cliente()

        self.codice_rapporto_to = 0
        self.codice_deposito_to = 0
        self.codice_fondo_to = ""
        self.descrizione_fondo_to = ""
        self.codice_tipocontratto_to = ""
        self.tipo_piano_to = ""
        self.numero_rate_piano = 0
        self.importo_rata = 0
        self.data_decorrenza = datetime.date.today().strftime('%Y-%m-%d')
        self.set_cliente()
        self.cognome = self.cliente['Cognome']
        self.nome = self.cliente['Nome']
        self.mifid = False

    def set_position_to_radrib(self, value):
        if not value >= 0:
            return "specificare valore"

        if not hasattr(self, 'cliente'):
            self.set_cliente()

        self.codice_rapporto_to = self.codice_rapporto_from
        self.codice_deposito_to = self.codice_deposito_from
        self.codice_fondo_to = self.codice_fondo_from
        self.descrizione_fondo_to = self.descrizione_fondo_from
        self.codice_tipocontratto_to = self.codice_tipocontratto_from
        self.tipo_piano_to = self.tipo_piano_from
        self.numero_rate_piano = 0
        self.importo_rata = 0
        self.cognome = self.cliente['Cognome']
        self.nome = self.cliente['Nome']
        self.mifid = False
        self.rad_rib_nop_pianis = value

    def set_position_to(self, index=None, cat=None):
        if not hasattr(self, 'cliente'):
            self.set_cliente()

        if self.tipo_operazione == "ATTCED":
            if not index >= 0:
                return "specificare indice"

            position = self.get_att_ced_positions_to()[index]

        elif self.tipo_operazione == "CARDIE":
            position = self.get_car_die_position_to()
            self.tipo_qta_movimento = "I"
            self.importo = 0
            self.numero_rate_versate = 0
            self.frequenza = ""
            self.data_decorrenza = datetime.date.today()
            self.fine_servizio = "R"
            self.fine_servizio_anno = 0
            self.fine_servizio_mese = 0
            self.rei_pro_perc_plus = 0
            self.rad_rib_flag = ""
            self.rad_rib_riduz_quota_a = 0
            self.rad_rib_riduz_quota_b = 0
            self.rad_rib_nop_pianis = 0
            self.rib_inv_perc_scost = 0
            self.iban = ""

        elif self.tipo_operazione == "RADRIB":
            # viene usato metodo set_position_to_radrib
            pass

        elif self.tipo_operazione == "REIPRO":
            if not index >= 0:
                return "specificare indice"

            # position = self.get_rei_pro_positions_to()[index]
            try:
                if cat == "ex":
                    position = self.get_swi_pro_ex_positions_to()[index]
                elif cat == "not":
                    position = self.get_swi_pro_not_positions_to()[index]
                else:
                    return "la categoria deve essere 'ex' o 'not'"
            except Exception, e:
                return e

        elif self.tipo_operazione == "RIBINV":
            position = self.get_rib_inv_position_to()
            self.tipo_qta_movimento = "I"
            self.importo = 0
            self.numero_rate_versate = 0
            self.frequenza = ""
            self.data_decorrenza = datetime.date.today()
            self.fine_servizio = "R"
            self.fine_servizio_anno = 0
            self.fine_servizio_mese = 0
            self.rei_pro_perc_plus = 0
            self.rad_rib_flag = ""
            self.rad_rib_riduz_quota_a = 0
            self.rad_rib_riduz_quota_b = 0
            self.rad_rib_nop_pianis = 0
            self.rib_inv_perc_scost = 0
            self.iban = ""

        elif self.tipo_operazione == "RIMPRO":
            position = self.get_rim_pro_position_to()

        elif self.tipo_operazione == "SWIPRO":
            if not index >= 0:
                return "specificare indice"

            try:
                if cat == "ex":
                    position = self.get_swi_pro_ex_positions_to()[index]
                elif cat == "not":
                    position = self.get_swi_pro_not_positions_to()[index]
                else:
                    return "la categoria deve essere 'ex' o 'not'"
            except Exception, e:
                return e

        self.codice_rapporto_to = position['RAPPORTO']
        self.codice_deposito_to = position['NUMCONTRATTO']
        self.codice_fondo_to = position['CODFONDO']
        self.descrizione_fondo_to = position['DESCRFONDO']
        self.codice_tipocontratto_to = position['CODTC']
        self.tipo_piano_to = position['TIPOPIANO']
        self.numero_rate_piano = position['NUMRATEPAC']
        self.importo_rata = position['IMPORTORATA']
        self.cognome = self.cliente['Cognome']
        self.nome = self.cliente['Nome']
        self.mifid = False

        return position

    def reset_position_to(self):
        #TODO: completare con nuovi campi
        if self.status == "FF":
            self.status = "IN"
            self.codice_rapporto_to = None
            self.codice_deposito_to = None
            self.codice_fondo_to = None
            self.descrizione_fondo_to = None
            self.codice_tipocontratto_to = None
            self.tipo_piano_to = None
            self.numero_rate_piano = None
            self.importo_rata = 0
            self.importo = 0
            self.quote = None
            self.iban = None
            self.numero_rate_versate = None
            self.tipo_qta_movimento = None

            self.cognome = None
            self.nome = None
            self.mifid = False

    def can_operate(self):
        mifid = 'N'
        if self.mifid:
            mifid = 'S'

        if not hasattr(self, 'op_servizio'):
            self.set_servizio()

        rad_rib_flag = 'N'
        if self.rad_rib_flag:
            rad_rib_flag = 'S'

        valore = self.importo
        if self.tipo_qta_movimento == "Q":
            valore = self.quote

        fine_servizio = ""
        if self.fine_servizio:
            fine_servizio = self.get_fine_servizio_display().upper()

        return self.op_servizio.op_servizi_check_can_operate(
            self.codice_fiscale, self.codice_cliente, self.tipo_operazione,
            self.codice_rapporto_from, self.codice_deposito_from, self.codice_fondo_from, self.codice_tipocontratto_from,
            self.codice_rapporto_to, self.codice_deposito_to, self.codice_fondo_to, self.codice_tipocontratto_to,
            self.get_tipo_qta_movimento_display(), valore, self.numero_rate_versate,
            mifid, self.iban,
            self.frequenza, self.data_decorrenza, fine_servizio, self.fine_servizio_anno, self.fine_servizio_mese,
            self.rei_pro_perc_plus, self.car_die_allocazione_a, self.car_die_allocazione_b,
            self.car_die_perc_var_a, self.car_die_perc_riall_ab, rad_rib_flag, self.rad_rib_riduz_quota_a,
            self.rad_rib_riduz_quota_b, self.rad_rib_nop_pianis, self.rib_inv_perc_scost, self.dettaglio_from,
            self.dettaglio_to, self.car_die_gruppo_a, self.car_die_gruppo_b
        )

    def save_servizio(self):
        if not self.can_operate()['OK']:
            return self.can_operate()

        if not hasattr(self, 'op_servizio'):
            self.set_servizio()

        mifid = 'N'
        if self.mifid:
            mifid = 'S'

        simulazione = 'S'
        if not self.simulazione:
            simulazione = 'N'

        rad_rib_flag = 'N'
        if self.rad_rib_flag:
            rad_rib_flag = 'S'

        fine_servizio = ""
        if self.fine_servizio:
            fine_servizio = self.get_fine_servizio_display().upper()

        bic = ""

        if self.tipo_operazione == "ATTCED":
            result = self.op_servizio.op_servizi_add_attiva_cedola(
                self.codice_cliente, self.codice_rapporto_from, self.codice_deposito_from, self.codice_fondo_from,
                self.codice_tipocontratto_from, self.cognome, self.nome, self.codice_fiscale,
                mifid, self.numero_rate_piano, self.importo_rata, simulazione,
                self.codice_rapporto_to, self.codice_deposito_to, self.codice_fondo_to, self.codice_tipocontratto_to,
                self.iban, bic
            )

        elif self.tipo_operazione == "CARDIE":
            result = self.op_servizio.op_servizi_add_carpe_diem(
                self.codice_cliente, self.codice_rapporto_from, self.codice_deposito_from, self.car_die_gruppo_a,
                self.codice_tipocontratto_from, self.cognome, self.nome, self.codice_fiscale,
                mifid, self.numero_rate_piano, self.importo_rata, simulazione,
                self.codice_rapporto_to, self.codice_deposito_to, self.car_die_gruppo_b, "",
                self.car_die_allocazione_a, self.car_die_allocazione_b,
                self.car_die_perc_var_a, self.car_die_perc_riall_ab, self.dettaglio_from
            )

        elif self.tipo_operazione == "RADRIB":
            result = self.op_servizio.op_servizi_add_raddoppia_ribasso(
                self.codice_cliente, self.codice_rapporto_from, self.codice_deposito_from, self.codice_fondo_from,
                self.codice_tipocontratto_from, self.cognome, self.nome, self.codice_fiscale,
                mifid, simulazione,
                self.frequenza, fine_servizio,
                self.fine_servizio_anno, self.fine_servizio_mese,
                self.rad_rib_riduz_quota_a, self.rad_rib_riduz_quota_b, self.rad_rib_nop_pianis
            )

        elif self.tipo_operazione == "REIPRO":
            result = self.op_servizio.op_servizi_add_reinvesti_profitti(
                self.codice_cliente, self.codice_rapporto_from, self.codice_deposito_from, self.codice_fondo_from,
                self.codice_tipocontratto_from, self.cognome, self.nome, self.codice_fiscale,
                mifid, self.numero_rate_piano, self.importo_rata, simulazione,
                self.codice_rapporto_to, self.codice_deposito_to, self.codice_fondo_to, self.codice_tipocontratto_to,
                self.frequenza, self.data_decorrenza, fine_servizio,
                self.fine_servizio_anno, self.fine_servizio_mese,
                self.rei_pro_perc_plus
            )

        elif self.tipo_operazione == "RIBINV":
            result = self.op_servizio.op_servizi_add_ribilancia_investimento(
                self.codice_cliente, self.codice_rapporto_from, self.codice_deposito_from, self.codice_fondo_from,
                self.codice_tipocontratto_from, self.cognome, self.nome, self.codice_fiscale,
                mifid, simulazione, fine_servizio,
                self.fine_servizio_anno, self.fine_servizio_mese,
                self.rib_inv_perc_scost, self.dettaglio_from, self.dettaglio_to
            )
        elif self.tipo_operazione == "RIMPRO":
            result = self.op_servizio.op_servizi_add_rimborso_programmato(
                self.codice_cliente, self.codice_rapporto_from, self.codice_deposito_from, self.codice_fondo_from,
                self.codice_tipocontratto_from, self.cognome, self.nome, self.codice_fiscale,
                mifid, self.quote, self.importo, self.numero_rate_piano, self.importo_rata, self.numero_rate_versate,
                simulazione, self.iban, bic, self.frequenza, self.data_decorrenza, fine_servizio,
                self.fine_servizio_anno, self.fine_servizio_mese
            )

        elif self.tipo_operazione == "SWIPRO":
            result = self.op_servizio.op_servizi_add_switch_programmato(
                self.codice_cliente, self.codice_rapporto_from, self.codice_deposito_from, self.codice_fondo_from,
                self.codice_tipocontratto_from, self.cognome, self.nome, self.codice_fiscale,
                mifid, self.quote, self.importo, self.numero_rate_piano, self.importo_rata, self.numero_rate_versate,
                simulazione, self.codice_rapporto_to, self.codice_deposito_to, self.codice_fondo_to,
                self.codice_tipocontratto_to,
                self.frequenza, self.data_decorrenza, fine_servizio, self.fine_servizio_anno, self.fine_servizio_mese,
                rad_rib_flag, self.rad_rib_riduz_quota_a, self.rad_rib_riduz_quota_b
            )

        else:
            result = _("%s: Tipo operazione inesistente" % self.tipo_operazione)
            return {'OK': False, 'numero_operazione': result}

        if isinstance(result, long) and not isinstance(result, dict):
            applicazione = Applicazione.objects.get(codice=self.tipo_operazione)
            # e' il numero dell'operazione
            self.status = "FN"
            self.data_operazione = datetime.datetime.now()
            self.messaggio_costo_applicazione = applicazione.get_pagamento_label(self.data_operazione.date())
            self.costo_applicazione = applicazione.costo_applicazione
            self.numero_operazione = result
            # ritorno un dizionario
            return {'OK': True, 'numero_operazione': result}
        return result

    class Meta(object):
        verbose_name = _("Servizio")
        verbose_name_plural = _("Servizi")

    def __unicode__(self):
        # return 'Servizio: ' + str(self.__dict__)
        return 'Servizio: %s' % self.id

if settings.CMS_ENABLED:
    import acomea.app.cms.plugins
